package com.ebaiyihui.alarm.server.service.Impl;

import com.ebaiyihui.alarm.server.config.ProjProperties;
import com.ebaiyihui.alarm.server.config.ServiceInfo;
import com.ebaiyihui.alarm.server.entity.AlarmMessage;
import com.ebaiyihui.alarm.server.service.AlarmService;
import com.ebaiyihui.alarm.server.util.HttpKit;
import com.ebaiyihui.framework.utils.StringUtil;
import com.google.gson.Gson;
import com.google.gson.JsonObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @description:
 * @author: yk
 * @create: 2023/08/09 09:52
 **/
@Service
public class AlarmServiceImpl implements AlarmService {

    private static final String ENDPOINT_RELATION = "ENDPOINT_RELATION";

    private static final String SERVICE = "SERVICE";

    @Value("${webHookUrl}")
    private String webHookUrl;

    @Value("${secret}")
    private String secret;

    @Autowired
    private ProjProperties projProperties;

    @Override
    public String receive(List<AlarmMessage> alarmMessages) {
        for (AlarmMessage alarmMessage : alarmMessages) {
            String name = alarmMessage.getName();
            String message = alarmMessage.getAlarmMessage();

            String scope = alarmMessage.getScope();
            // 过滤一下不发送服务通知逻辑
            if (SERVICE.equals(scope)) {
                if (Objects.nonNull(projProperties.getNotNotifyService()) && projProperties.getNotNotifyService().contains(name)) {
                    return "success";
                }
            }
            // 过滤一下不发送服务通知接口逻辑
            if (ENDPOINT_RELATION.equalsIgnoreCase(scope)) {
                // 获取路径地址
                String tempName = name.substring(name.indexOf("/"), name.length());
                String pathUrl = tempName.substring(0, tempName.indexOf(" "));
                if (Objects.nonNull(projProperties.getNotNotifyService()) && projProperties.getNotNotifyUrl().contains(pathUrl)) {
                    return "success";
                }
            }

            // 获取服务名称
            String serviceNme = name.substring(name.indexOf("byh"), name.length());
            List<ServiceInfo> serviceInfos = projProperties.getServiceInfo();
            for (ServiceInfo serviceInfo : serviceInfos) {
                // 相应的服务通知对应人员
                if (serviceInfo.getServiceNames().contains(serviceNme)) {
                    //  message += "<at user_id=\\\"ou_6cb2f35f9bf8e7d7e977483801ab13de\\\">杨凯</at>";
                    message += "<at user_id=\\\"" + serviceInfo.getServiceNotifyName() + "\\\"></at>";
                    alarmMessage.setAlarmMessage(message);
                }
                String sendMessage = "{\"msg_type\":\"text\",\"content\":{\"text\":\"Apache SkyWalking Alarm: \\n %s.\"}}";
                String requestBody = getRequestBody(sendMessage, alarmMessage);
                try {
                    HttpKit.jsonPost(webHookUrl, requestBody);
                } catch (Exception e) {
                    return "fail";
                }
            }

        }
        return "success";
    }

    /**
     * deal requestBody,if it has sign set the sign
     */
    private String getRequestBody(String sendMessage, AlarmMessage alarmMessage) {
        String requestBody = String.format(sendMessage, alarmMessage.getAlarmMessage());
        Gson gson = new Gson();
        JsonObject jsonObject = gson.fromJson(requestBody, JsonObject.class);
        Map<String, Object> content = buildContent(jsonObject);
        if (!StringUtil.isBlank(secret)) {
            Long timestamp = System.currentTimeMillis() / 1000;
            content.put("timestamp", timestamp);
            try {
                content.put("sign", sign(timestamp, secret));
            } catch (NoSuchAlgorithmException | InvalidKeyException e) {
                throw new RuntimeException(e);
            }
        }
        return gson.toJson(content);
    }

    /**
     * build content,if it has ats someone set the ats
     */
    private Map<String, Object> buildContent(JsonObject jsonObject) {
        Map<String, Object> content = new HashMap<String, Object>();
        content.put("msg_type", jsonObject.get("msg_type").getAsString());
        if (jsonObject.get("ats") != null) {
            String ats = jsonObject.get("ats").getAsString();
            List<String> collect = Arrays.stream(ats.split(",")).map(String::trim).collect(Collectors.toList());
            String text = jsonObject.get("content").getAsJsonObject().get("text").getAsString();
            for (final String userId : collect) {
                text += "<at user_id=\"" + userId + "\"></at>";
            }
            jsonObject.get("content").getAsJsonObject().addProperty("text", text);
        }
        content.put("content", jsonObject.get("content").getAsJsonObject());
        return content;
    }

    /**
     * Sign webhook url using HmacSHA256 algorithm
     */
    private String sign(final Long timestamp, String secret) throws NoSuchAlgorithmException, InvalidKeyException {
        String stringToSign = timestamp + "\n" + secret;
        Mac mac = Mac.getInstance("HmacSHA256");
        mac.init(new SecretKeySpec(stringToSign.getBytes(), "HmacSHA256"));
        byte[] signData = mac.doFinal();
        return Base64.getEncoder().encodeToString(signData);
    }
}
